const dotenv = require('dotenv');
const express = require('express');
const mongoose = require('mongoose');
const morgan = require('morgan');

// init express
dotenv.config(); // You can define the port here, but Docker will take care of it
const app = express();
const PORT = process.env.PORT || 8080;

// connect to mongoose
const MONGODB_URI = process.env.MONGODB_URI;

mongoose.connect(MONGODB_URI, {
  useNewUrlParser: true,
  useFindAndModify: false,
  useUnifiedTopology: true
});

mongoose.connection.on('connected', () => {
  console.log('Mongoose is connected');
});

// HTTP request logger in the terminal
app.use(morgan('tiny'));

// Let Express use JSON bodies
app.use(express.json());
app.use(express.urlencoded({ extended: false }));

// Route imports
const user = require('./routes/userRoutes');

// Route config
app.use('/api/user', user);

// Default test route
app.get('*', (req, res) => {
  res.send({ msg: 'success' });
});

app.listen(PORT, console.log(`Server is starting at port ${PORT}`));
