const axios = require('axios');
const express = require('express');
const { Expo } = require('expo-server-sdk');
const router = express.Router();
const User = require('../models/user');

router.get('/get', (req, res) => {
  let id = -1;
  if( req.body.id === undefined ) {
    id = req.query.id;
  } else {
    id = req.body.id;
  }

  const query = User.where({uid: id});

  query.findOne(function (err, result) {
    if(err) {
      console.log(err);
      res.status(500).send(err);
    } else if (result) {
      console.log(result);
      res.json(result);
    } else {
      res.status(404).send({"msg": "no results"});
    }
  });
})

router.post('/create', (req, res) => {
  let id = -1;
  let followingList = [];
  let expoPushToken = '';

  if( req.body.id === undefined ) {
    id = req.query.id;
    followingList = req.query.followingList;
    expoPushToken = req.query.expoPushToken;
  } else {
    id = req.body.id;
    followingList = req.body.followingList;
    expoPushToken = req.body.expoPushToken;
  }

  console.log(id);

  const data = {
    uid: id,
    followingList: followingList,
    expoPushToken: expoPushToken,
  };

  const newUser = new User(data);

  newUser.save((err) => {
    if(err) {
      res.status(500).send(err);
    } else {
      res.status(201).send({'msg': 'success'});
    }
  })
})

router.patch('/follow', (req, res) => {
  let id = -1;
  let followingList = [];

  if(req.body.id === undefined ) {
    id = req.query.id;
    followingList = req.query.followingList;
  } else {
    id = req.body.id;
    followingList = req.body.followingList;
  }

  /* can be used to check if some stuff is missing
  if (expoPushToken === undefined || followingList === undefined) {
    res.status(406).send({"msg": "incorrect formattign"});
  }
  */
  const query = User.where({ uid: id });
  query.findOne(function (err, result) {
    if(err) {
      console.log(err);
      res.send(err);
    } else if (result) {
      query.updateOne({ uid: id }, { $push: {followingList: followingList}}, (errInner, resInner) => {
        if (errInner) {
          console.log(errInner);
          res.send(errInner);
        } else {
          console.log(resInner);
          res.send(resInner)
        }
      })
    } else {
      res.status(404).send({"msg": "no user found"})
    }
  })
})

router.patch('/unfollow', (req, res) => {
  let id = -1;
  let tempArgs = undefined;
  let followingList = [];

  if(req.body.id === undefined) {
    tempArgs = req.query;
  } else {
    tempArgs = req.body;
  }

  console.log(tempArgs);
  id = tempArgs.id;
  followingList = tempArgs.unfollow;

  // This method of querying and getting request data is a lot better/quicker to implement
  User.updateOne( {uid: id}, {$pullAll: {followingList: [followingList]}}, (err, resQuery) => {
    if (resQuery) {
      res.send(resQuery);
    } else if (err) {
      res.status(404).send(err);
    } else {
      res.status(404).send({"msg": "no user found"})
    }
  })
})

router.patch('/updatePushToken', (req, res) => {
  let id = -1;
  let tempArgs = undefined;

  if (req.body.id === undefined) {
    tempArgs = req.query;
  } else {
    tempArgs = req.body;
  }

  id = tempArgs.id;
  const expoPushToken = tempArgs.expoPushToken;

  if (expoPushToken === undefined) {
    res.status(406).send({"msg": "include expo push token"});
    return;
  }

  User.updateOne( { uid: id }, { $set: { expoPushToken: expoPushToken }}, (err, resQuery) => {
    if (err) {
      res.send(err);
    } else {
      res.send(resQuery);
    }
  })
})

router.post('/notify', (req, res) => {
  let tempArgs = req.body;
  if (req.body.id === undefined) {
    tempArgs = req.query;
  }

  const notify = tempArgs.id;
  const username = tempArgs.username;
  let title = `${username} has commented on your story`;

  if (tempArgs.isAnonNotif === true) {
    title = "A user has commented on your story";
  }

  const message = "come check it out";
  const storyId = tempArgs.storyId;
  const isAnonNotif = tempArgs.isAnonNotif;

  const notification = {
    username: username,
    title: title,
    message: message,
    storyId: storyId,
    isAnonNotif: isAnonNotif,
  };

  const headers = {
    'host': 'exp.host',
    'accept': 'application/json',
    'accept-encoding': 'gzip, deflate',
    'content-type': 'application/json'
  };

  const query = User.where({ uid: notify });
  query.findOne(function (err, result) {
    if(err) {
      res.send(err);
    } else if (result) {
      query.updateOne({ uid: notify }, { $push: {notificationList: notification}}, (errInner, resInner) => {
        if (errInner) {
          res.send(errInner);
        } else {

          console.log('expoPushToken: ', result.expoPushToken);
          const expoNotif = {
            to: result.expoPushToken,
            title: title,
            body: message,
          };

          axios.post('https://exp.host/--/api/v2/push/send', expoNotif, headers)
            .then((exRes) => {
              console.log(exRes.data);
            }).catch((err) => {
              console.log(err);
            })

          res.send(resInner);
        }
      })
    } else {
      res.status(404).send({"msg": "no user found"})
    }
  })

})

module.exports = router;
